@@ -1,5 +1,5 @@
 # engine/__init__.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -83,7 +83,7 @@ default_strategy = "plain"
 
 
 def create_engine(*args, **kwargs):
-    """Create a new :class:`.Engine` instance.
+    """Create a new :class:`_engine.Engine` instance.
 
     The standard calling form is to send the URL as the
     first positional argument, usually a string
@@ -93,8 +93,8 @@ def create_engine(*args, **kwargs):
         engine = create_engine("postgresql://scott:tiger@localhost/test")
 
     Additional keyword arguments may then follow it which
-    establish various options on the resulting :class:`.Engine`
-    and its underlying :class:`.Dialect` and :class:`.Pool`
+    establish various options on the resulting :class:`_engine.Engine`
+    and its underlying :class:`.Dialect` and :class:`_pool.Pool`
     constructs::
 
         engine = create_engine("mysql://scott:tiger@hostname/dbname",
@@ -109,17 +109,19 @@ def create_engine(*args, **kwargs):
 
     ``**kwargs`` takes a wide variety of options which are routed
     towards their appropriate components.  Arguments may be specific to
-    the :class:`.Engine`, the underlying :class:`.Dialect`, as well as the
-    :class:`.Pool`.  Specific dialects also accept keyword arguments that
+    the :class:`_engine.Engine`, the underlying :class:`.Dialect`,
+    as well as the
+    :class:`_pool.Pool`.  Specific dialects also accept keyword arguments that
     are unique to that dialect.   Here, we describe the parameters
-    that are common to most :func:`.create_engine()` usage.
+    that are common to most :func:`_sa.create_engine()` usage.
 
-    Once established, the newly resulting :class:`.Engine` will
-    request a connection from the underlying :class:`.Pool` once
-    :meth:`.Engine.connect` is called, or a method which depends on it
-    such as :meth:`.Engine.execute` is invoked.   The :class:`.Pool` in turn
+    Once established, the newly resulting :class:`_engine.Engine` will
+    request a connection from the underlying :class:`_pool.Pool` once
+    :meth:`_engine.Engine.connect` is called, or a method which depends on it
+    such as :meth:`_engine.Engine.execute` is invoked.   The
+    :class:`_pool.Pool` in turn
     will establish the first actual DBAPI connection when this request
-    is received.   The :func:`.create_engine` call itself does **not**
+    is received.   The :func:`_sa.create_engine` call itself does **not**
     establish any actual DBAPI connections directly.
 
     .. seealso::
@@ -149,7 +151,7 @@ def create_engine(*args, **kwargs):
 
         .. deprecated:: 1.3
 
-            The :paramref:`.create_engine.convert_unicode` parameter
+            The :paramref:`_sa.create_engine.convert_unicode` parameter
             is deprecated and will be removed in a future release.
             All modern DBAPIs now support Python Unicode directly and this
             parameter is unnecessary.
@@ -160,6 +162,18 @@ def create_engine(*args, **kwargs):
         connections. Usage of this function causes connection
         parameters specified in the URL argument to be bypassed.
 
+        This hook is not as flexible as the newer
+        :class:`_events.DialectEvents.do_connect` hook which allows complete
+        control over how a connection is made to the database, given the full
+        set of URL arguments and state beforehand.
+
+        .. seealso::
+
+            :class:`_events.DialectEvents.do_connect` - event hook that allows
+            full control over DBAPI connection mechanics.
+
+            :ref:`custom_dbapi_args`
+
     :param echo=False: if True, the Engine will log all statements
         as well as a ``repr()`` of their parameter lists to the default log
         handler, which defaults to ``sys.stdout`` for output.   If set to the
@@ -173,6 +187,7 @@ def create_engine(*args, **kwargs):
             :ref:`dbengine_logging` - further detail on how to configure
             logging.
 
+
     :param echo_pool=False: if True, the connection pool will log
         informational output such as when connections are invalidated
         as well as when connections are recycled to the default log handler,
@@ -208,25 +223,30 @@ def create_engine(*args, **kwargs):
     :param encoding: Defaults to ``utf-8``.  This is the string
         encoding used by SQLAlchemy for string encode/decode
         operations which occur within SQLAlchemy, **outside of
-        the DBAPI.**  Most modern DBAPIs feature some degree of
-        direct support for Python ``unicode`` objects,
-        what you see in Python 2 as a string of the form
-        ``u'some string'``.  For those scenarios where the
-        DBAPI is detected as not supporting a Python ``unicode``
-        object, this encoding is used to determine the
-        source/destination encoding.  It is **not used**
-        for those cases where the DBAPI handles unicode
-        directly.
+        the DBAPIs own encoding facilities.**
 
-        To properly configure a system to accommodate Python
-        ``unicode`` objects, the DBAPI should be
-        configured to handle unicode to the greatest
-        degree as is appropriate - see
-        the notes on unicode pertaining to the specific
-        target database in use at :ref:`dialect_toplevel`.
+        .. note:: The ``encoding`` parameter deals only with in-Python
+           encoding issues that were prevalent with many DBAPIs under  Python
+           2.  Under Python 3 it is mostly unused.   For  DBAPIs that require
+           client encoding configurations, such as those of MySQL and Oracle,
+           please consult specific :ref:`dialect documentation
+           <dialect_toplevel>` for details.
+
+        All modern DBAPIs that work in Python 3 necessarily feature direct
+        support for Python unicode strings.   Under Python 2, this was not
+        always the case.  For those scenarios where the DBAPI is detected as
+        not supporting a Python ``unicode`` object under Python 2, this
+        encoding is used to determine the source/destination encoding.  It is
+        **not used** for those cases where the DBAPI handles unicode directly.
+
+        To properly configure a system to accommodate Python ``unicode``
+        objects, the DBAPI should be configured to handle unicode to the
+        greatest degree as is appropriate - see the notes on unicode pertaining
+        to the specific target database in use at :ref:`dialect_toplevel`.
 
         Areas where string encoding may need to be accommodated
-        outside of the DBAPI include zero or more of:
+        outside of the DBAPI, nearly always under **Python 2 only**,
+        include zero or more of:
 
         * the values passed to bound parameters, corresponding to
           the :class:`.Unicode` type or the :class:`.String` type
@@ -242,13 +262,12 @@ def create_engine(*args, **kwargs):
         * the string column names retrieved from the DBAPI's
           ``cursor.description`` attribute.
 
-        When using Python 3, the DBAPI is required to support
-        *all* of the above values as Python ``unicode`` objects,
-        which in Python 3 are just known as ``str``.  In Python 2,
-        the DBAPI does not specify unicode behavior at all,
-        so SQLAlchemy must make decisions for each of the above
-        values on a per-DBAPI basis - implementations are
-        completely inconsistent in their behavior.
+        When using Python 3, the DBAPI is required to support all of the above
+        values as Python ``unicode`` objects, which in Python 3 are just known
+        as ``str``.  In Python 2, the DBAPI does not specify unicode behavior
+        at all, so SQLAlchemy must make decisions for each of the above values
+        on a per-DBAPI basis - implementations are completely inconsistent in
+        their behavior.
 
     :param execution_options: Dictionary execution options which will
         be applied to all connections.  See
@@ -260,6 +279,11 @@ def create_engine(*args, **kwargs):
 
         .. versionadded:: 1.3.8
 
+        .. seealso::
+
+            :ref:`dbengine_logging` - further detail on how to configure
+            logging.
+
     :param implicit_returning=True: When ``True``, a RETURNING-
         compatible construct, if available, will be used to
         fetch newly generated primary key values when a single row
@@ -272,22 +296,23 @@ def create_engine(*args, **kwargs):
     :param isolation_level: this string parameter is interpreted by various
         dialects in order to affect the transaction isolation level of the
         database connection.   The parameter essentially accepts some subset of
-        these string arguments: ``"SERIALIZABLE"``, ``"REPEATABLE_READ"``,
-        ``"READ_COMMITTED"``, ``"READ_UNCOMMITTED"`` and ``"AUTOCOMMIT"``.
+        these string arguments: ``"SERIALIZABLE"``, ``"REPEATABLE READ"``,
+        ``"READ COMMITTED"``, ``"READ UNCOMMITTED"`` and ``"AUTOCOMMIT"``.
         Behavior here varies per backend, and
         individual dialects should be consulted directly.
 
         Note that the isolation level can also be set on a
-        per-:class:`.Connection` basis as well, using the
+        per-:class:`_engine.Connection` basis as well, using the
         :paramref:`.Connection.execution_options.isolation_level`
         feature.
 
         .. seealso::
 
-            :attr:`.Connection.default_isolation_level` - view default level
+            :attr:`_engine.Connection.default_isolation_level`
+            - view default level
 
             :paramref:`.Connection.execution_options.isolation_level`
-            - set per :class:`.Connection` isolation level
+            - set per :class:`_engine.Connection` isolation level
 
             :ref:`SQLite Transaction Isolation <sqlite_isolation_level>`
 
@@ -297,7 +322,8 @@ def create_engine(*args, **kwargs):
 
             :ref:`session_transaction_isolation` - for the ORM
 
-    :param json_deserializer: for dialects that support the :class:`.JSON`
+    :param json_deserializer: for dialects that support the
+        :class:`_types.JSON`
         datatype, this is a Python callable that will convert a JSON string
         to a Python object.  By default, the Python ``json.loads`` function is
         used.
@@ -305,7 +331,7 @@ def create_engine(*args, **kwargs):
         .. versionchanged:: 1.3.7  The SQLite dialect renamed this from
            ``_json_deserializer``.
 
-    :param json_serializer: for dialects that support the :class:`.JSON`
+    :param json_serializer: for dialects that support the :class:`_types.JSON`
         datatype, this is a Python callable that will render a given object
         as JSON.   By default, the Python ``json.dumps`` function is used.
 
@@ -317,14 +343,15 @@ def create_engine(*args, **kwargs):
         characters. If less than 6, labels are generated as
         "_(counter)". If ``None``, the value of
         ``dialect.max_identifier_length``, which may be affected via the
-        :paramref:`.create_engine.max_identifier_length` parameter,
-        is used instead.   The value of :paramref:`.create_engine.label_length`
+        :paramref:`_sa.create_engine.max_identifier_length` parameter,
+        is used instead.   The value of
+        :paramref:`_sa.create_engine.label_length`
         may not be larger than that of
-        :paramref:`.create_engine.max_identfier_length`.
+        :paramref:`_sa.create_engine.max_identfier_length`.
 
         .. seealso::
 
-            :paramref:`.create_engine.max_identifier_length`
+            :paramref:`_sa.create_engine.max_identifier_length`
 
     :param listeners: A list of one or more
         :class:`~sqlalchemy.interfaces.PoolListener` objects which will
@@ -335,6 +362,13 @@ def create_engine(*args, **kwargs):
         "sqlalchemy.engine" logger. Defaults to a hexstring of the
         object's id.
 
+        .. seealso::
+
+            :ref:`dbengine_logging` - further detail on how to configure
+            logging.
+
+
+
     :param max_identifier_length: integer; override the max_identifier_length
         determined by the dialect.  if ``None`` or zero, has no effect.  This
         is the database's configured maximum number of characters that may be
@@ -348,7 +382,7 @@ def create_engine(*args, **kwargs):
 
         .. seealso::
 
-            :paramref:`.create_engine.label_length`
+            :paramref:`_sa.create_engine.label_length`
 
     :param max_overflow=10: the number of connections to allow in
         connection pool "overflow", that is connections that can be
@@ -361,7 +395,7 @@ def create_engine(*args, **kwargs):
         specific DBAPI which will be imported before first connect.  This
         parameter causes the import to be bypassed, and the given module to
         be used instead. Can be used for testing of DBAPIs as well as to
-        inject "mock" DBAPI implementations into the :class:`.Engine`.
+        inject "mock" DBAPI implementations into the :class:`_engine.Engine`.
 
     :param paramstyle=None: The `paramstyle <http://legacy.python.org/dev/peps/pep-0249/#paramstyle>`_
         to use when rendering bound parameters.  This style defaults to the
@@ -394,6 +428,13 @@ def create_engine(*args, **kwargs):
        "sqlalchemy.pool" logger. Defaults to a hexstring of the object's
        id.
 
+
+       .. seealso::
+
+            :ref:`dbengine_logging` - further detail on how to configure
+            logging.
+
+
     :param pool_pre_ping: boolean, if True will enable the connection pool
         "pre-ping" feature that tests connections for liveness upon
         each checkout.
@@ -426,13 +467,13 @@ def create_engine(*args, **kwargs):
             :ref:`pool_setting_recycle`
 
     :param pool_reset_on_return='rollback': set the
-        :paramref:`.Pool.reset_on_return` parameter of the underlying
-        :class:`.Pool` object, which can be set to the values
+        :paramref:`_pool.Pool.reset_on_return` parameter of the underlying
+        :class:`_pool.Pool` object, which can be set to the values
         ``"rollback"``, ``"commit"``, or ``None``.
 
         .. seealso::
 
-            :paramref:`.Pool.reset_on_return`
+            :paramref:`_pool.Pool.reset_on_return`
 
     :param pool_timeout=30: number of seconds to wait before giving
         up on getting a connection from the pool. This is only used
@@ -488,7 +529,7 @@ def engine_from_config(configuration, prefix="sqlalchemy.", **kwargs):
     ``sqlalchemy.url``, ``sqlalchemy.echo``, etc.  The 'prefix' argument
     indicates the prefix to be searched for.  Each matching key (after the
     prefix is stripped) is treated as though it were the corresponding keyword
-    argument to a :func:`.create_engine` call.
+    argument to a :func:`_sa.create_engine` call.
 
     The only required key is (assuming the default prefix) ``sqlalchemy.url``,
     which provides the :ref:`database URL <database_urls>`.
