@@ -1,5 +1,5 @@
 # orm/deprecated_interfaces.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -12,7 +12,7 @@ from .. import util
 
 @util.langhelpers.dependency_for("sqlalchemy.orm.interfaces")
 class MapperExtension(object):
-    """Base implementation for :class:`.Mapper` event hooks.
+    """Base implementation for :class:`_orm.Mapper` event hooks.
 
     .. deprecated:: 0.7
 
@@ -398,7 +398,7 @@ class SessionExtension(object):
         """Execute before flush process has started.
 
         `instances` is an optional list of objects which were passed to
-        the ``flush()`` method. """
+        the ``flush()`` method."""
 
     def after_flush(self, session, flush_context):
         """Execute after flush has completed, but before commit has been
@@ -415,18 +415,18 @@ class SessionExtension(object):
         This will be when the 'new', 'dirty', and 'deleted' lists are in
         their final state.  An actual commit() may or may not have
         occurred, depending on whether or not the flush started its own
-        transaction or participated in a larger transaction. """
+        transaction or participated in a larger transaction."""
 
     def after_begin(self, session, transaction, connection):
         """Execute after a transaction is begun on a connection
 
         `transaction` is the SessionTransaction. This method is called
-        after an engine level transaction is begun on a connection. """
+        after an engine level transaction is begun on a connection."""
 
     def after_attach(self, session, instance):
         """Execute after an instance is attached to a session.
 
-        This is called after an add, delete or merge. """
+        This is called after an add, delete or merge."""
 
     def after_bulk_update(self, session, query, query_context, result):
         """Execute after a bulk update operation to the session.
@@ -464,7 +464,7 @@ class AttributeExtension(object):
     remove, and append events on individual mapped attributes.
     It is established on an individual mapped attribute using
     the `extension` argument, available on
-    :func:`.column_property`, :func:`.relationship`, and
+    :func:`.column_property`, :func:`_orm.relationship`, and
     others::
 
         from sqlalchemy.orm.interfaces import AttributeExtension
@@ -501,7 +501,7 @@ class AttributeExtension(object):
     even if it means firing lazy callables.
 
     Note that ``active_history`` can also be set directly via
-    :func:`.column_property` and :func:`.relationship`.
+    :func:`.column_property` and :func:`_orm.relationship`.
 
     """
 
