@@ -12,7 +12,6 @@ from sqlalchemy import MetaData
 from sqlalchemy import or_
 from sqlalchemy import PrimaryKeyConstraint
 from sqlalchemy import select
-from sqlalchemy import Sequence
 from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import testing
@@ -28,6 +27,7 @@ from sqlalchemy.testing.assertsql import CursorSQL
 from sqlalchemy.testing.assertsql import DialectSQL
 from sqlalchemy.util import ue
 
+
 metadata = None
 cattable = None
 matchtable = None
@@ -244,34 +244,31 @@ class QueryUnicodeTest(fixtures.TestBase):
 
     @testing.requires.mssql_freetds
     @testing.requires.python2
+    @testing.provide_metadata
     def test_convert_unicode(self):
-        meta = MetaData(testing.db)
+        meta = self.metadata
         t1 = Table(
             "unitest_table",
             meta,
             Column("id", Integer, primary_key=True),
-            Column("descr", mssql.MSText(convert_unicode=True)),
+            Column("descr", mssql.MSText()),
         )
         meta.create_all()
-        con = testing.db.connect()
-
-        # encode in UTF-8 (sting object) because this is the default
-        # dialect encoding
-
-        con.execute(
-            ue(
-                "insert into unitest_table values ('bien u\
-                    umang\xc3\xa9')"
-            ).encode("UTF-8")
-        )
-        try:
-            r = t1.select().execute().first()
-            assert isinstance(r[1], util.text_type), (
-                "%s is %s instead of unicode, working on %s"
-                % (r[1], type(r[1]), meta.bind)
+        with testing.db.connect() as con:
+            con.execute(
+                ue(
+                    "insert into unitest_table values ('abc \xc3\xa9 def')"
+                ).encode("UTF-8")
             )
-        finally:
-            meta.drop_all()
+            r = con.execute(t1.select()).first()
+            assert isinstance(
+                r[1], util.text_type
+            ), "%s is %s instead of unicode, working on %s" % (
+                r[1],
+                type(r[1]),
+                meta.bind,
+            )
+            eq_(r[1], util.ue("abc \xc3\xa9 def"))
 
 
 class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
@@ -314,7 +311,7 @@ class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
         t1 = Table(
             "t1",
             meta,
-            Column("id", Integer, Sequence("fred", 100, 1), primary_key=True),
+            Column("id", Integer, mssql_identity_start=100, primary_key=True),
             Column("descr", String(200)),
             # the following flag will prevent the
             # MSSQLCompiler.returning_clause from getting called,
@@ -326,7 +323,7 @@ class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
         t2 = Table(
             "t2",
             meta,
-            Column("id", Integer, Sequence("fred", 200, 1), primary_key=True),
+            Column("id", Integer, mssql_identity_start=200, primary_key=True),
             Column("descr", String(200)),
         )
         meta.create_all()
@@ -362,7 +359,8 @@ class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
         metadata.create_all(engine)
 
         with self.sql_execution_asserter(engine) as asserter:
-            engine.execute(t1.insert(), {"data": "somedata"})
+            with engine.begin() as conn:
+                conn.execute(t1.insert(), {"data": "somedata"})
 
         # TODO: need a dialect SQL that acts like Cursor SQL
         asserter.assert_(
@@ -385,7 +383,8 @@ class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
         metadata.create_all(engine)
 
         with self.sql_execution_asserter(engine) as asserter:
-            engine.execute(t1.insert())
+            with engine.begin() as conn:
+                conn.execute(t1.insert())
 
         # even with pyodbc, we don't embed the scope identity on a
         # DEFAULT VALUES insert
@@ -409,7 +408,8 @@ class QueryTest(testing.AssertsExecutionResults, fixtures.TestBase):
         metadata.create_all(engine)
 
         with self.sql_execution_asserter(engine) as asserter:
-            engine.execute(t1.insert(), {"data": "somedata"})
+            with engine.begin() as conn:
+                conn.execute(t1.insert(), {"data": "somedata"})
 
         # pyodbc-specific system
         asserter.assert_(
