@@ -30,8 +30,9 @@ from sqlalchemy import UniqueConstraint
 from sqlalchemy import util
 from sqlalchemy.dialects.mysql import base as mysql
 from sqlalchemy.dialects.mysql import reflection as _reflection
+from sqlalchemy.schema import CreateIndex
 from sqlalchemy.testing import assert_raises_message
-from sqlalchemy.testing import AssertsExecutionResults
+from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
@@ -125,7 +126,10 @@ class TypeReflectionTest(fixtures.TestBase):
             (mysql.YEAR(display_width=4), mysql.YEAR(display_width=4)),
         ]
 
-        self._run_test(specs, ["display_width"])
+        if testing.against("mysql>=8.0.19"):
+            self._run_test(specs, [])
+        else:
+            self._run_test(specs, ["display_width"])
 
     def test_string_types(self):
         specs = [
@@ -192,7 +196,14 @@ class TypeReflectionTest(fixtures.TestBase):
                 (BigInteger, mysql.BIGINT(display_width=20)),
             ]
         )
-        self._run_test(specs, ["display_width", "unsigned", "zerofill"])
+
+        # TODO: mysql 8.0.19-ish doesn't consistently report
+        # on display_width.   need to test this more accurately though
+        # for the cases where it does
+        if testing.against("mysql >= 8.0.19"):
+            self._run_test(specs, ["unsigned", "zerofill"])
+        else:
+            self._run_test(specs, ["display_width", "unsigned", "zerofill"])
 
     def test_binary_types(self):
         specs = [
@@ -216,7 +227,7 @@ class TypeReflectionTest(fixtures.TestBase):
         self._run_test(specs, ["enums"])
 
 
-class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
+class ReflectionTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __only_on__ = "mysql"
     __backend__ = True
@@ -224,6 +235,9 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
     def test_default_reflection(self):
         """Test reflection of column defaults."""
 
+        # TODO: this test is a mess.   should be broken into individual
+        # combinations
+
         from sqlalchemy.dialects.mysql import VARCHAR
 
         def_table = Table(
@@ -248,6 +262,8 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                     )
                 ),
             ),
+            Column("c7", mysql.DOUBLE(), DefaultClause("0.0000")),
+            Column("c8", mysql.DOUBLE(22, 6), DefaultClause("0.0000")),
         )
         def_table.create()
         try:
@@ -267,6 +283,15 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         assert reflected.c.c5.default is None
         assert reflected.c.c5.server_default is None
         assert reflected.c.c6.default is None
+        assert str(reflected.c.c7.server_default.arg) in ("0", "'0'")
+
+        # this is because the numeric is 6 decimal places, MySQL
+        # formats it to that many places.
+        assert str(reflected.c.c8.server_default.arg) in (
+            "0.000000",
+            "'0.000000'",
+        )
+
         assert re.match(
             r"CURRENT_TIMESTAMP(\(\))? ON UPDATE CURRENT_TIMESTAMP(\(\))?",
             str(reflected.c.c6.server_default.arg).upper(),
@@ -287,6 +312,11 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         assert reflected.c.c5.default is None
         assert reflected.c.c5.server_default is None
         assert reflected.c.c6.default is None
+        assert str(reflected.c.c7.server_default.arg) in ("0", "'0'")
+        assert str(reflected.c.c8.server_default.arg) in (
+            "0.000000",
+            "'0.000000'",
+        )
         assert re.match(
             r"CURRENT_TIMESTAMP(\(\))? ON UPDATE CURRENT_TIMESTAMP(\(\))?",
             str(reflected.c.c6.server_default.arg).upper(),
@@ -692,6 +722,57 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         self.assert_(indexes["uc_a"].unique)
         self.assert_("uc_a" not in constraints)
 
+    @testing.provide_metadata
+    def test_reflect_fulltext(self):
+        mt = Table(
+            "mytable",
+            self.metadata,
+            Column("id", Integer, primary_key=True),
+            Column("textdata", String(50)),
+            mysql_engine="InnoDB",
+        )
+        Index("textdata_ix", mt.c.textdata, mysql_prefix="FULLTEXT")
+        self.metadata.create_all(testing.db)
+
+        mt = Table("mytable", MetaData(), autoload_with=testing.db)
+        idx = list(mt.indexes)[0]
+        eq_(idx.name, "textdata_ix")
+        eq_(idx.dialect_options["mysql"]["prefix"], "FULLTEXT")
+        self.assert_compile(
+            CreateIndex(idx),
+            "CREATE FULLTEXT INDEX textdata_ix ON mytable (textdata)",
+        )
+
+    @testing.requires.mysql_ngram_fulltext
+    @testing.provide_metadata
+    def test_reflect_fulltext_comment(self):
+        mt = Table(
+            "mytable",
+            self.metadata,
+            Column("id", Integer, primary_key=True),
+            Column("textdata", String(50)),
+            mysql_engine="InnoDB",
+        )
+        Index(
+            "textdata_ix",
+            mt.c.textdata,
+            mysql_prefix="FULLTEXT",
+            mysql_with_parser="ngram",
+        )
+
+        self.metadata.create_all(testing.db)
+
+        mt = Table("mytable", MetaData(), autoload_with=testing.db)
+        idx = list(mt.indexes)[0]
+        eq_(idx.name, "textdata_ix")
+        eq_(idx.dialect_options["mysql"]["prefix"], "FULLTEXT")
+        eq_(idx.dialect_options["mysql"]["with_parser"], "ngram")
+        self.assert_compile(
+            CreateIndex(idx),
+            "CREATE FULLTEXT INDEX textdata_ix ON mytable "
+            "(textdata) WITH PARSER ngram",
+        )
+
     @testing.provide_metadata
     def test_non_column_index(self):
         m1 = self.metadata
@@ -707,12 +788,12 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             [{"name": "foo_idx", "column_names": ["x"], "unique": False}],
         )
 
-    def _bug_88718_casing_0(self):
+    def _bug_88718_96365_casing_0(self):
         fkeys_casing_0 = [
             {
                 "name": "FK_PlaylistTTrackId",
                 "constrained_columns": ["TTrackID"],
-                "referred_schema": "test_schema",
+                "referred_schema": "Test_Schema",
                 "referred_table": "Track",
                 "referred_columns": ["trackid"],
                 "options": {},
@@ -727,17 +808,17 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             },
         ]
         ischema_casing_0 = [
-            ("test", "Track", "TrackID"),
-            ("test_schema", "Track", "TrackID"),
+            ("Test", "Track", "TrackID"),
+            ("Test_Schema", "Track", "TrackID"),
         ]
         return fkeys_casing_0, ischema_casing_0
 
-    def _bug_88718_casing_1(self):
+    def _bug_88718_96365_casing_1(self):
         fkeys_casing_1 = [
             {
                 "name": "FK_PlaylistTTrackId",
                 "constrained_columns": ["TTrackID"],
-                "referred_schema": "test_schema",
+                "referred_schema": "Test_Schema",
                 "referred_table": "Track",
                 "referred_columns": ["trackid"],
                 "options": {},
@@ -752,18 +833,20 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             },
         ]
         ischema_casing_1 = [
-            (util.u("test"), util.u("Track"), "TrackID"),
-            (util.u("test_schema"), util.u("Track"), "TrackID"),
+            (util.u("Test"), util.u("Track"), "TrackID"),
+            (util.u("Test_Schema"), util.u("Track"), "TrackID"),
         ]
         return fkeys_casing_1, ischema_casing_1
 
-    def _bug_88718_casing_2(self):
+    def _bug_88718_96365_casing_2(self):
         fkeys_casing_2 = [
             {
                 "name": "FK_PlaylistTTrackId",
                 "constrained_columns": ["TTrackID"],
+                # I haven't tested schema name but since col/table both
+                # do it, assume schema name also comes back wrong
                 "referred_schema": "test_schema",
-                "referred_table": "Track",
+                "referred_table": "track",
                 "referred_columns": ["trackid"],
                 "options": {},
             },
@@ -771,38 +854,40 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                 "name": "FK_PlaylistTrackId",
                 "constrained_columns": ["TrackID"],
                 "referred_schema": None,
-                "referred_table": "Track",
+                # table name also comes back wrong (probably schema also)
+                # with casing=2, see https://bugs.mysql.com/bug.php?id=96365
+                "referred_table": "track",
                 "referred_columns": ["trackid"],
                 "options": {},
             },
         ]
         ischema_casing_2 = [
-            ("test", "Track", "TrackID"),
-            ("test_schema", "Track", "TrackID"),
+            ("Test", "Track", "TrackID"),
+            ("Test_Schema", "Track", "TrackID"),
         ]
         return fkeys_casing_2, ischema_casing_2
 
-    def test_correct_for_mysql_bug_88718(self):
+    def test_correct_for_mysql_bugs_88718_96365(self):
         dialect = mysql.dialect()
 
         for casing, (fkeys, ischema) in [
-            (0, self._bug_88718_casing_0()),
-            (1, self._bug_88718_casing_1()),
-            (2, self._bug_88718_casing_2()),
+            (0, self._bug_88718_96365_casing_0()),
+            (1, self._bug_88718_96365_casing_1()),
+            (2, self._bug_88718_96365_casing_2()),
         ]:
             dialect._casing = casing
-            dialect.default_schema_name = "test"
+            dialect.default_schema_name = "Test"
             connection = mock.Mock(
                 dialect=dialect, execute=lambda stmt, **params: ischema
             )
-            dialect._correct_for_mysql_bug_88718(fkeys, connection)
+            dialect._correct_for_mysql_bugs_88718_96365(fkeys, connection)
             eq_(
                 fkeys,
                 [
                     {
                         "name": "FK_PlaylistTTrackId",
                         "constrained_columns": ["TTrackID"],
-                        "referred_schema": "test_schema",
+                        "referred_schema": "Test_Schema",
                         "referred_table": "Track",
                         "referred_columns": ["TrackID"],
                         "options": {},
@@ -858,6 +943,14 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         m1.create_all()
 
         if testing.db.dialect._casing in (1, 2):
+            # the original test for the 88718 fix here in [ticket:4344]
+            # actually set  referred_table='track', with the wrong casing!
+            # this test was never run. with [ticket:4751], I've gone through
+            # the trouble to create docker containers with true
+            # lower_case_table_names=2 and per
+            # https://bugs.mysql.com/bug.php?id=96365 the table name being
+            # lower case is also an 8.0 regression.
+
             eq_(
                 inspect(testing.db).get_foreign_keys("PlaylistTrack"),
                 [
@@ -865,7 +958,7 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                         "name": "FK_PlaylistTTrackId",
                         "constrained_columns": ["TTrackID"],
                         "referred_schema": testing.config.test_schema,
-                        "referred_table": "track",
+                        "referred_table": "Track",
                         "referred_columns": ["TrackID"],
                         "options": {},
                     },
@@ -873,7 +966,7 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                         "name": "FK_PlaylistTrackId",
                         "constrained_columns": ["TrackID"],
                         "referred_schema": None,
-                        "referred_table": "track",
+                        "referred_table": "Track",
                         "referred_columns": ["TrackID"],
                         "options": {},
                     },
@@ -881,7 +974,10 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             )
         else:
             eq_(
-                inspect(testing.db).get_foreign_keys("PlaylistTrack"),
+                sorted(
+                    inspect(testing.db).get_foreign_keys("PlaylistTrack"),
+                    key=lambda elem: elem["name"],
+                ),
                 [
                     {
                         "name": "FK_PlaylistTTrackId",
@@ -998,6 +1094,12 @@ class RawReflectionTest(fixtures.TestBase):
         assert regex.match(
             "  KEY (`id`) USING BTREE COMMENT 'prefix''text''suffix'"
         )
+        # https://forums.mysql.com/read.php?20,567102,567111#msg-567111
+        # "It means if the MySQL version >= 501, execute what's in the comment"
+        assert regex.match(
+            "  FULLTEXT KEY `ix_fulltext_oi_g_name` (`oi_g_name`) "
+            "/*!50100 WITH PARSER `ngram` */ "
+        )
 
     def test_key_reflection_columns(self):
         regex = self.parser._re_key
