@@ -1,7 +1,6 @@
 import sqlalchemy as tsa
 from sqlalchemy import create_engine
 from sqlalchemy import event
-from sqlalchemy import exc
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import String
@@ -373,22 +372,6 @@ class DDLEventTest(fixtures.TestBase):
         )
         eq_(metadata_canary.mock_calls, [])
 
-    def test_append_listener(self):
-        metadata, table, bind = self.metadata, self.table, self.bind
-
-        def fn(*a):
-            return None
-
-        table.append_ddl_listener("before-create", fn)
-        assert_raises(
-            exc.InvalidRequestError, table.append_ddl_listener, "blah", fn
-        )
-
-        metadata.append_ddl_listener("before-create", fn)
-        assert_raises(
-            exc.InvalidRequestError, metadata.append_ddl_listener, "blah", fn
-        )
-
 
 class DDLExecutionTest(fixtures.TestBase):
     def setup(self):
@@ -466,66 +449,6 @@ class DDLExecutionTest(fixtures.TestBase):
         assert "xyzzy" in strings
         assert "fnord" in strings
 
-    def test_deprecated_append_ddl_listener_table(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
-        canary = []
-        users.append_ddl_listener(
-            "before-create", lambda e, t, b: canary.append("mxyzptlk")
-        )
-        users.append_ddl_listener(
-            "after-create", lambda e, t, b: canary.append("klptzyxm")
-        )
-        users.append_ddl_listener(
-            "before-drop", lambda e, t, b: canary.append("xyzzy")
-        )
-        users.append_ddl_listener(
-            "after-drop", lambda e, t, b: canary.append("fnord")
-        )
-
-        metadata.create_all()
-        assert "mxyzptlk" in canary
-        assert "klptzyxm" in canary
-        assert "xyzzy" not in canary
-        assert "fnord" not in canary
-        del engine.mock[:]
-        canary[:] = []
-        metadata.drop_all()
-        assert "mxyzptlk" not in canary
-        assert "klptzyxm" not in canary
-        assert "xyzzy" in canary
-        assert "fnord" in canary
-
-    def test_deprecated_append_ddl_listener_metadata(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
-        canary = []
-        metadata.append_ddl_listener(
-            "before-create",
-            lambda e, t, b, tables=None: canary.append("mxyzptlk"),
-        )
-        metadata.append_ddl_listener(
-            "after-create",
-            lambda e, t, b, tables=None: canary.append("klptzyxm"),
-        )
-        metadata.append_ddl_listener(
-            "before-drop", lambda e, t, b, tables=None: canary.append("xyzzy")
-        )
-        metadata.append_ddl_listener(
-            "after-drop", lambda e, t, b, tables=None: canary.append("fnord")
-        )
-
-        metadata.create_all()
-        assert "mxyzptlk" in canary
-        assert "klptzyxm" in canary
-        assert "xyzzy" not in canary
-        assert "fnord" not in canary
-        del engine.mock[:]
-        canary[:] = []
-        metadata.drop_all()
-        assert "mxyzptlk" not in canary
-        assert "klptzyxm" not in canary
-        assert "xyzzy" in canary
-        assert "fnord" in canary
-
     def test_metadata(self):
         metadata, engine = self.metadata, self.engine
 
@@ -572,7 +495,7 @@ class DDLExecutionTest(fixtures.TestBase):
         assert "fnord" in strings
 
     def test_conditional_constraint(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
+        metadata, users = self.metadata, self.users
         nonpg_mock = engines.mock_engine(dialect_name="sqlite")
         pg_mock = engines.mock_engine(dialect_name="postgresql")
         constraint = CheckConstraint(
@@ -609,7 +532,7 @@ class DDLExecutionTest(fixtures.TestBase):
 
     @testing.uses_deprecated(r".*use the DDLEvents")
     def test_conditional_constraint_deprecated(self):
-        metadata, users, engine = self.metadata, self.users, self.engine
+        metadata, users = self.metadata, self.users
         nonpg_mock = engines.mock_engine(dialect_name="sqlite")
         pg_mock = engines.mock_engine(dialect_name="postgresql")
         constraint = CheckConstraint(
@@ -645,31 +568,32 @@ class DDLExecutionTest(fixtures.TestBase):
         table = self.users
         ddl = DDL("SELECT 1")
 
-        for py in (
-            "engine.execute(ddl)",
-            "engine.execute(ddl, table)",
-            "cx.execute(ddl)",
-            "cx.execute(ddl, table)",
-            "ddl.execute(engine)",
-            "ddl.execute(engine, table)",
-            "ddl.execute(cx)",
-            "ddl.execute(cx, table)",
+        for spec in (
+            (engine.execute, ddl),
+            (engine.execute, ddl, table),
+            (cx.execute, ddl),
+            (cx.execute, ddl, table),
+            (ddl.execute, engine),
+            (ddl.execute, engine, table),
+            (ddl.execute, cx),
+            (ddl.execute, cx, table),
         ):
-            r = eval(py)
-            assert list(r) == [(1,)], py
+            fn = spec[0]
+            arg = spec[1:]
+            r = fn(*arg)
+            eq_(list(r), [(1,)])
 
-        for py in ("ddl.execute()", "ddl.execute(target=table)"):
-            try:
-                r = eval(py)
-                assert False
-            except tsa.exc.UnboundExecutionError:
-                pass
+        for fn, kw in ((ddl.execute, {}), (ddl.execute, dict(target=table))):
+            assert_raises(tsa.exc.UnboundExecutionError, fn, **kw)
 
         for bind in engine, cx:
             ddl.bind = bind
-            for py in ("ddl.execute()", "ddl.execute(target=table)"):
-                r = eval(py)
-                assert list(r) == [(1,)], py
+            for fn, kw in (
+                (ddl.execute, {}),
+                (ddl.execute, dict(target=table)),
+            ):
+                r = fn(**kw)
+                eq_(list(r), [(1,)])
 
     def test_platform_escape(self):
         """test the escaping of % characters in the DDL construct."""
@@ -779,27 +703,3 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
             )
             ._should_execute(tbl, cx)
         )
-
-    @testing.uses_deprecated(r"See DDLEvents")
-    def test_filter_deprecated(self):
-        cx = self.mock_engine()
-
-        tbl = Table("t", MetaData(), Column("id", Integer))
-        target = cx.name
-
-        assert DDL("")._should_execute_deprecated("x", tbl, cx)
-        assert DDL("", on=target)._should_execute_deprecated("x", tbl, cx)
-        assert not DDL("", on="bogus")._should_execute_deprecated("x", tbl, cx)
-        assert DDL("", on=lambda d, x, y, z: True)._should_execute_deprecated(
-            "x", tbl, cx
-        )
-        assert DDL(
-            "", on=lambda d, x, y, z: z.engine.name != "bogus"
-        )._should_execute_deprecated("x", tbl, cx)
-
-    def test_repr(self):
-        assert repr(DDL("s"))
-        assert repr(DDL("s", on="engine"))
-        assert repr(DDL("s", on=lambda x: 1))
-        assert repr(DDL("s", context={"a": 1}))
-        assert repr(DDL("s", on="engine", context={"a": 1}))
