@@ -7,6 +7,7 @@ from sqlalchemy.orm import mapper
 from sqlalchemy.orm import Session
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import mock
 from sqlalchemy.testing.assertsql import CompiledSQL
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
@@ -80,7 +81,7 @@ class BulkInsertUpdateTest(BulkTest, _fixtures.FixtureTest):
         mapper(Order, o)
 
     def test_bulk_save_return_defaults(self):
-        User, = self.classes("User")
+        (User,) = self.classes("User")
 
         s = Session()
         objects = [User(name="u1"), User(name="u2"), User(name="u3")]
@@ -102,8 +103,64 @@ class BulkInsertUpdateTest(BulkTest, _fixtures.FixtureTest):
         )
         eq_(objects[0].__dict__["id"], 1)
 
+    def test_bulk_save_mappings_preserve_order(self):
+        (User,) = self.classes("User")
+
+        s = Session()
+
+        # commit some object into db
+        user1 = User(name="i1")
+        user2 = User(name="i2")
+        s.add(user1)
+        s.add(user2)
+        s.commit()
+
+        # make some changes
+        user1.name = "u1"
+        user3 = User(name="i3")
+        s.add(user3)
+        user2.name = "u2"
+
+        objects = [user1, user3, user2]
+
+        from sqlalchemy import inspect
+
+        def _bulk_save_mappings(
+            mapper,
+            mappings,
+            isupdate,
+            isstates,
+            return_defaults,
+            update_changed_only,
+            render_nulls,
+        ):
+            mock_method(list(mappings), isupdate)
+
+        mock_method = mock.Mock()
+        with mock.patch.object(s, "_bulk_save_mappings", _bulk_save_mappings):
+            s.bulk_save_objects(objects)
+            eq_(
+                mock_method.mock_calls,
+                [
+                    mock.call([inspect(user1)], True),
+                    mock.call([inspect(user3)], False),
+                    mock.call([inspect(user2)], True),
+                ],
+            )
+
+        mock_method = mock.Mock()
+        with mock.patch.object(s, "_bulk_save_mappings", _bulk_save_mappings):
+            s.bulk_save_objects(objects, preserve_order=False)
+            eq_(
+                mock_method.mock_calls,
+                [
+                    mock.call([inspect(user3)], False),
+                    mock.call([inspect(user1), inspect(user2)], True),
+                ],
+            )
+
     def test_bulk_save_no_defaults(self):
-        User, = self.classes("User")
+        (User,) = self.classes("User")
 
         s = Session()
         objects = [User(name="u1"), User(name="u2"), User(name="u3")]
@@ -121,7 +178,7 @@ class BulkInsertUpdateTest(BulkTest, _fixtures.FixtureTest):
         assert "id" not in objects[0].__dict__
 
     def test_bulk_save_updated_include_unchanged(self):
-        User, = self.classes("User")
+        (User,) = self.classes("User")
 
         s = Session(expire_on_commit=False)
         objects = [User(name="u1"), User(name="u2"), User(name="u3")]
@@ -147,7 +204,7 @@ class BulkInsertUpdateTest(BulkTest, _fixtures.FixtureTest):
         )
 
     def test_bulk_update(self):
-        User, = self.classes("User")
+        (User,) = self.classes("User")
 
         s = Session(expire_on_commit=False)
         objects = [User(name="u1"), User(name="u2"), User(name="u3")]
@@ -177,7 +234,7 @@ class BulkInsertUpdateTest(BulkTest, _fixtures.FixtureTest):
         )
 
     def test_bulk_insert(self):
-        User, = self.classes("User")
+        (User,) = self.classes("User")
 
         s = Session()
         with self.sql_execution_asserter() as asserter:
@@ -202,7 +259,7 @@ class BulkInsertUpdateTest(BulkTest, _fixtures.FixtureTest):
         )
 
     def test_bulk_insert_render_nulls(self):
-        Order, = self.classes("Order")
+        (Order,) = self.classes("Order")
 
         s = Session()
         with self.sql_execution_asserter() as asserter:
