@@ -266,10 +266,13 @@ class NaturalPKTest(fixtures.MappedTest):
             # test passive_updates=True; update user
             self.assert_sql_count(testing.db, go, 1)
         sess.expunge_all()
-        assert User(
-            username="jack",
-            addresses=[Address(username="jack"), Address(username="jack")],
-        ) == sess.query(User).get("jack")
+        assert (
+            User(
+                username="jack",
+                addresses=[Address(username="jack"), Address(username="jack")],
+            )
+            == sess.query(User).get("jack")
+        )
 
         u1 = sess.query(User).get("jack")
         u1.addresses = []
@@ -1117,10 +1120,13 @@ class NonPKCascadeTest(fixtures.MappedTest):
             # test passive_updates=True; update user
             self.assert_sql_count(testing.db, go, 1)
         sess.expunge_all()
-        assert User(
-            username="jack",
-            addresses=[Address(username="jack"), Address(username="jack")],
-        ) == sess.query(User).get(u1.id)
+        assert (
+            User(
+                username="jack",
+                addresses=[Address(username="jack"), Address(username="jack")],
+            )
+            == sess.query(User).get(u1.id)
+        )
         sess.expunge_all()
 
         u1 = sess.query(User).get(u1.id)
@@ -1612,7 +1618,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
         self._test_fk_threelevel(False)
 
     def _test_pk(self, passive_updates):
-        Engineer, = self.classes("Engineer")
+        (Engineer,) = self.classes("Engineer")
         self._mapping_fixture(False, passive_updates)
         sess = sa.orm.sessionmaker()()
 
@@ -1685,7 +1691,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
         )
 
     def _test_pk_threelevel(self, passive_updates):
-        Owner, = self.classes("Owner")
+        (Owner,) = self.classes("Owner")
 
         self._mapping_fixture(True, passive_updates)
 
